@@ -108,6 +108,8 @@ static const ff_codec_t ff_audio_lookup[] = {
   {BUF_AUDIO_SMACKER,    CODEC_ID_SMACKAUDIO,     "Smacker (ffmpeg)"},
   {BUF_AUDIO_FLVADPCM,   CODEC_ID_ADPCM_SWF,	  "Flash ADPCM (ffmpeg)"},
   {BUF_AUDIO_WAVPACK,	 CODEC_ID_WAVPACK,	  "WavPack (ffmpeg)"},
+  {BUF_AUDIO_AMR_NB,	 CODEC_ID_AMR_NB,	  "AMR narrow band (ffmpeg)"},
+  {BUF_AUDIO_AMR_WB,	 CODEC_ID_AMR_WB,	  "AMR wide band (ffmpeg)"},
 };
 
 
@@ -178,7 +180,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
           this->context->bit_rate    = audio_header->nAvgBytesPerSec * 8;
       
           if(audio_header->cbSize > 0) {
-            this->context->extradata = xine_xmalloc(audio_header->cbSize);
+            this->context->extradata = malloc(audio_header->cbSize);
             this->context->extradata_size = audio_header->cbSize;
             memcpy( this->context->extradata, 
                     (uint8_t *)audio_header + sizeof(xine_waveformatex),
@@ -203,7 +205,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
             this->context->block_align = _X_BE_32(&this->buf[0x18]);
 
             this->context->extradata_size = 5*sizeof(short);
-            this->context->extradata      = xine_xmalloc(this->context->extradata_size);
+            this->context->extradata      = malloc(this->context->extradata_size);
   
             ptr = (short *) this->context->extradata;
   
@@ -249,8 +251,8 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
                 break; /* abort early - extradata length is bad */
 
 	      this->context->extradata_size = data_len;
-	      this->context->extradata      = xine_xmalloc(this->context->extradata_size +
-							   FF_INPUT_BUFFER_PADDING_SIZE);
+	      this->context->extradata      = malloc(this->context->extradata_size +
+						     FF_INPUT_BUFFER_PADDING_SIZE);
 	      xine_fast_memcpy (this->context->extradata, this->buf + extradata,
 				this->context->extradata_size);
 	      break;
@@ -276,7 +278,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
   
       this->size = 0;
   
-      this->decode_buffer = xine_xmalloc(AVCODEC_MAX_AUDIO_FRAME_SIZE);
+      this->decode_buffer = calloc(1, AVCODEC_MAX_AUDIO_FRAME_SIZE);
   
       return;
     }
@@ -284,8 +286,8 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
              (buf->decoder_info[1] == BUF_SPECIAL_STSD_ATOM)) {
 
     this->context->extradata_size = buf->decoder_info[2];
-    this->context->extradata = xine_xmalloc(buf->decoder_info[2] +
-      FF_INPUT_BUFFER_PADDING_SIZE);
+    this->context->extradata = malloc(buf->decoder_info[2] +
+				      FF_INPUT_BUFFER_PADDING_SIZE);
     memcpy(this->context->extradata, buf->decoder_info_ptr[2],
       buf->decoder_info[2]);
 
@@ -450,7 +452,7 @@ static audio_decoder_t *ff_audio_open_plugin (audio_decoder_class_t *class_gen,
 
   ff_audio_decoder_t *this ;
 
-  this = (ff_audio_decoder_t *) xine_xmalloc (sizeof (ff_audio_decoder_t));
+  this = calloc(1, sizeof (ff_audio_decoder_t));
 
   this->audio_decoder.decode_data         = ff_audio_decode_data;
   this->audio_decoder.reset               = ff_audio_reset;
@@ -486,7 +488,7 @@ void *init_audio_plugin (xine_t *xine, void *data) {
 
   ff_audio_class_t *this ;
 
-  this = (ff_audio_class_t *) xine_xmalloc (sizeof (ff_audio_class_t));
+  this = calloc(1, sizeof (ff_audio_class_t));
 
   this->decoder_class.open_plugin     = ff_audio_open_plugin;
   this->decoder_class.get_identifier  = ff_audio_get_identifier;
@@ -598,7 +600,12 @@ static uint32_t supported_audio_types[] = {
   #ifdef CONFIG_WAVPACK_DECODER
   BUF_AUDIO_WAVPACK,
   #endif
-  
+  #ifdef CONFIG_AMR_NB_DECODER
+  BUF_AUDIO_AMR_NB,
+  #endif
+  #ifdef CONFIG_AMR_WB_DECODER
+  BUF_AUDIO_AMR_WB,
+  #endif
   0
 };
 
