@@ -321,6 +321,7 @@ static const ff_codec_t ff_video_lookup[] = {
   {BUF_VIDEO_CAVS,        CODEC_ID_CAVS,       "Chinese AVS (ffmpeg)"},
   {BUF_VIDEO_VMNC,        CODEC_ID_VMNC,       "VMware Screen Codec (ffmpeg)"},
   {BUF_VIDEO_THEORA_RAW,  CODEC_ID_THEORA,     "Theora (ffmpeg)"},
+  {BUF_VIDEO_SNOW,        CODEC_ID_SNOW,       "Snow (ffmpeg)"},
 };
 
 static const char *const skip_loop_filter_enum_names[] = {
@@ -1892,6 +1893,9 @@ static uint32_t supported_video_types[] = {
   #ifdef CONFIG_VMNC_DECODER
   BUF_VIDEO_VMNC,
   #endif
+  #ifdef CONFIG_SNOW_DECODER
+  BUF_VIDEO_SNOW,
+  #endif
   BUF_VIDEO_THEORA_RAW,
   0 
 };
