@@ -33,6 +33,9 @@
 #ifdef HAVE_ALLOCA_H
 #include <alloca.h>
 #endif
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
 
 #define LOG_MODULE "demux_dts"
 #define LOG_VERBOSE
@@ -392,7 +395,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_dts_t   *this;
 
-  this         = xine_xmalloc (sizeof (demux_dts_t));
+  this         = calloc(1, sizeof(demux_dts_t));
   this->stream = stream;
   this->input  = input;
 
@@ -464,7 +467,7 @@ static void class_dispose (demux_class_t *this_gen) {
 void *demux_dts_init_plugin (xine_t *xine, void *data) {
   demux_dts_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_dts_class_t));
+  this = calloc(1, sizeof(demux_dts_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
