@@ -15,12 +15,10 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /*
- * $Id: demux_mng.c,v 1.29 2007/01/19 00:26:40 dgp85 Exp $
- *
  * demux_mng.c, Demuxer plugin for Multiple-image Network Graphics format
  *
  * written and currently maintained by Robin Kay <komadori@myrealbox.com>
@@ -118,7 +116,9 @@ static mng_bool mymng_process_header(mng_handle mngh, mng_uint32 width, mng_uint
   this->bih.biHeight = height;
   this->left_edge = (this->bih.biWidth - width) / 2;
 
-  this->image = malloc(this->bih.biWidth * height * 3);
+  this->image = malloc((mng_size_t)this->bih.biWidth * (mng_size_t)height * 3);
+  if (!this->image)
+    return MNG_FALSE;
 
   mng_set_canvasstyle(mngh, MNG_CANVAS_RGB8);
 
@@ -262,7 +262,7 @@ static demux_plugin_t* open_plugin(demux_class_t *class_gen, xine_stream_t *stre
 
   demux_mng_t    *this;
 
-  this         = xine_xmalloc (sizeof (demux_mng_t));
+  this         = calloc(1, sizeof(demux_mng_t));
   this->stream = stream;
   this->input  = input;
 
@@ -362,7 +362,7 @@ static void class_dispose(demux_class_t *this){
 static void *init_plugin(xine_t *xine, void *data){
   demux_mng_class_t     *this;
 
-  this  = xine_xmalloc (sizeof (demux_mng_class_t));
+  this  = calloc(1, sizeof(demux_mng_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
