@@ -15,13 +15,10 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: demux_mpeg_block.c,v 1.220 2007/02/20 00:34:55 dgp85 Exp $
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * demultiplexer for mpeg 1/2 program streams
  * used with fixed blocksize devices (like dvd/vcd)
- *
  */
  
 #ifdef HAVE_CONFIG_H
@@ -1374,7 +1371,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   input_plugin_t     *input = (input_plugin_t *) input_gen;
   demux_mpeg_block_t *this;
 
-  this         = xine_xmalloc (sizeof (demux_mpeg_block_t));
+  this         = calloc(1, sizeof(demux_mpeg_block_t));
   this->stream = stream;
   this->input  = input;
   
@@ -1543,7 +1540,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   demux_mpeg_block_class_t     *this;
 
-  this         = xine_xmalloc (sizeof (demux_mpeg_block_class_t));
+  this         = calloc(1, sizeof(demux_mpeg_block_class_t));
   this->config = xine->config;
   this->xine   = xine;
 
