@@ -50,7 +50,6 @@
 #define FOURCC_TAG BE_FOURCC
 #define NSVf_TAG       FOURCC_TAG('N', 'S', 'V', 'f')
 #define NSVs_TAG       FOURCC_TAG('N', 'S', 'V', 's')
-#define NONE_TAG       FOURCC_TAG('N', 'O', 'N', 'E')
 
 #define BEEF 0xEFBE
 
@@ -256,17 +255,11 @@ static int open_nsv_file(demux_nsv_t *this) {
     return 0;
 
   /* check for a 'NSV' signature */
-  if ((preview[0] != 'N') ||
-      (preview[1] != 'S') ||
-      (preview[2] != 'V'))
-  {
-    if ((preview[0] != 'Z') ||
-        (preview[1] != 0)   ||
-	(preview[2] != '9'))
-      return 0;
+  if ( memcmp(preview, "Z\09", 3) == 0) {
     this->is_ultravox = preview[3];
     this->ultravox_first = 1;
-  }
+  } else if ( memcmp(preview, "NSV", 3) != 0 )
+    return 0;
 
   lprintf("NSV file detected, ultravox=%d\n", this->is_ultravox);
 
@@ -302,13 +295,13 @@ static int open_nsv_file(demux_nsv_t *this) {
 	return 0;
       
       this->video_fourcc = _X_ME_32(&preview[4]);
-      if (_X_BE_32(&preview[4]) == NONE_TAG)
+      if (_x_is_fourcc(&preview[4], "NONE"))
 	this->video_type = 0;
       else
 	this->video_type = _x_fourcc_to_buf_video(this->video_fourcc);
       
       this->audio_fourcc = _X_ME_32(&preview[8]);
-      if (_X_BE_32(&preview[8]) == NONE_TAG)
+      if (_x_is_fourcc(&preview[8], "NONE"))
 	this->audio_type = 0;
       else
 	this->audio_type = _x_formattag_to_buf_audio(this->audio_fourcc);
@@ -594,7 +587,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_nsv_t    *this;
 
-  this         = xine_xmalloc (sizeof (demux_nsv_t));
+  this         = calloc(1, sizeof(demux_nsv_t));
   this->stream = stream;
   this->input  = input;
 
@@ -668,7 +661,7 @@ static void class_dispose (demux_class_t *this_gen) {
 static void *demux_nsv_init_plugin (xine_t *xine, void *data) {
   demux_nsv_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_nsv_class_t));
+  this = calloc(1, sizeof(demux_nsv_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
