@@ -72,7 +72,7 @@ static int open_tta_file(demux_tta_t *this) {
   if (_x_demux_read_header(this->input, peek, 4) != 4)
       return 0;
 
-  if ( _X_BE_32(peek) != FOURCC_32('T', 'T', 'A', '1') )
+  if ( !_x_is_fourcc(peek, "TTA1") )
     return 0;
 
   if ( this->input->read(this->input, this->header.buffer, sizeof(this->header)) != sizeof(this->header) )
@@ -233,7 +233,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   demux_tta_t    *this;
 
-  this         = xine_xmalloc (sizeof (demux_tta_t));
+  this         = calloc(1, sizeof(demux_tta_t));
   this->stream = stream;
   this->input  = input;
 
@@ -307,7 +307,7 @@ static void class_dispose (demux_class_t *this_gen) {
 void *demux_tta_init_plugin (xine_t *xine, void *data) {
   demux_tta_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_tta_class_t));
+  this = calloc(1, sizeof(demux_tta_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
