@@ -111,9 +111,6 @@ static int find_chunk_by_tag(demux_wav_t *this, const uint32_t given_chunk_tag,
 /* returns 1 if the WAV file was opened successfully, 0 otherwise */
 static int open_wav_file(demux_wav_t *this) {
   uint8_t signature[WAV_SIGNATURE_SIZE];
-  uint32_t chunk_tag;
-  uint32_t chunk_size;
-  uint8_t chunk_preamble[8];
   off_t wave_pos;
 
   /* check the signature */
@@ -129,7 +126,7 @@ static int open_wav_file(demux_wav_t *this) {
     return 0;
 
   this->input->seek(this->input, wave_pos, SEEK_SET);
-  this->wave = xine_xmalloc( this->wave_size );
+  this->wave = malloc( this->wave_size );
 
   if (!this->wave || this->input->read(this->input, (void *)this->wave, this->wave_size) !=
     this->wave_size) {
@@ -360,7 +357,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   demux_wav_t    *this;
   uint32_t	align;
 
-  this         = xine_xmalloc (sizeof (demux_wav_t));
+  this         = calloc(1, sizeof(demux_wav_t));
   this->stream = stream;
   this->input  = input;
 
@@ -446,7 +443,7 @@ static void class_dispose (demux_class_t *this_gen) {
 void *demux_wav_init_plugin (xine_t *xine, void *data) {
   demux_wav_class_t     *this;
 
-  this = xine_xmalloc (sizeof (demux_wav_class_t));
+  this = calloc(1, sizeof(demux_wav_class_t));
 
   this->demux_class.open_plugin     = open_plugin;
   this->demux_class.get_description = get_description;
