@@ -401,7 +401,7 @@ static input_plugin_t *mms_class_get_instance (input_class_t *cls_gen, xine_stre
     return NULL;
   }
 
-  this = (mms_input_plugin_t *) xine_xmalloc (sizeof (mms_input_plugin_t));
+  this = calloc(1, sizeof (mms_input_plugin_t));
   cls->ip = this;
   this->stream   = stream;
   this->mms      = NULL;
@@ -459,7 +459,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   mms_input_class_t  *this;
 
-  this = (mms_input_class_t *) xine_xmalloc (sizeof (mms_input_class_t));
+  this = calloc(1, sizeof (mms_input_class_t));
 
   this->xine   = xine;
   this->ip                             = NULL;
