@@ -82,8 +82,6 @@ static off_t pnm_plugin_read (input_plugin_t *this_gen,
 
   lprintf ("pnm_plugin_read: %"PRId64" bytes ...\n", len);
 
-  nbc_check_buffers (this->nbc);
-
   n = pnm_read (this->pnm, buf, len);
   this->curpos += n;
 
@@ -231,7 +229,7 @@ static input_plugin_t *pnm_class_get_instance (input_class_t *cls_gen, xine_stre
     return NULL;
   }
 
-  this = (pnm_input_plugin_t *) xine_xmalloc (sizeof (pnm_input_plugin_t));
+  this = calloc(1, sizeof (pnm_input_plugin_t));
 
   this->stream = stream;
   this->pnm    = NULL;
@@ -276,7 +274,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   pnm_input_class_t  *this;
 
-  this = (pnm_input_class_t *) xine_xmalloc (sizeof (pnm_input_class_t));
+  this = calloc(1, sizeof (pnm_input_class_t));
 
   this->xine   = xine;
 
