@@ -17,7 +17,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * pvr input plugin for WinTV-PVR 250/350 pci cards using driver from:
  *   http://ivtv.sf.net
@@ -37,8 +37,6 @@
  *
  * usage: 
  *   xine pvr:/<prefix_to_tmp_files>\!<prefix_to_saved_files>\!<max_page_age>
- *
- * $Id: input_pvr.c,v 1.65 2007/01/19 01:05:25 dgp85 Exp $
  */
 
 /**************************************************************************
@@ -378,10 +376,10 @@ static void pvrscr_exit (scr_plugin_t *scr) {
   free(this);
 }
 
-static pvrscr_t* pvrscr_init (void) {
+static pvrscr_t *XINE_MALLOC pvrscr_init (void) {
   pvrscr_t *this;
 
-  this = (pvrscr_t *) xine_xmalloc(sizeof(pvrscr_t));
+  this = calloc(1, sizeof(pvrscr_t));
 
   this->scr.interface_version = 3;
   this->scr.get_priority      = pvrscr_get_priority;
@@ -506,34 +504,24 @@ static void pvr_adjust_realtime_speed(pvr_input_plugin_t *this, fifo_buffer_t *f
 }
 
 #define PVR_FILENAME      "%s%08d_%08d.vob"
-#define PVR_FILENAME_SIZE 1+8+1+8+4+1
 
 static char *make_temp_name(pvr_input_plugin_t *this, int page) {
-
   char *filename;
   
-  int size = strlen(this->tmp_prefix)+PVR_FILENAME_SIZE;
-  filename = malloc(size);
-      
-  snprintf(filename, size, PVR_FILENAME, this->tmp_prefix, this->session, page);
+  asprintf(&filename, PVR_FILENAME, this->tmp_prefix, this->session, page);
   
   return filename;
 }
       
 #define SAVE_BASE_FILENAME     "ch%03d %02d-%02d-%04d %02d:%02d:%02d"
-#define SAVE_BASE_FILENAME_SIZE 2+3+1+2+1+2+1+4+1+2+1+2+1+2+1
       
 static char *make_base_save_name(int channel, time_t tm) {
-  
   struct tm rec_time;
   char *filename;
   
-  int size = SAVE_BASE_FILENAME_SIZE;
-  filename = malloc(size);
-  
   localtime_r(&tm, &rec_time);
       
-  snprintf(filename, size, SAVE_BASE_FILENAME, 
+  asprintf(&filename, SAVE_BASE_FILENAME, 
            channel, rec_time.tm_mon+1, rec_time.tm_mday,
            rec_time.tm_year+1900, rec_time.tm_hour, rec_time.tm_min,
            rec_time.tm_sec);
@@ -541,16 +529,11 @@ static char *make_base_save_name(int channel, time_t tm) {
 }
 
 #define SAVE_FILENAME      "%s%s_%04d.vob"
-#define SAVE_FILENAME_SIZE 1+4+4+1
 
 static char *make_save_name(pvr_input_plugin_t *this, char *base, int page) {
-
   char *filename;
   
-  int size = strlen(this->save_prefix)+strlen(base)+SAVE_FILENAME_SIZE;
-  filename = malloc(size);
-      
-  snprintf(filename, size, SAVE_FILENAME, this->save_prefix, base, page);
+  asprintf(&filename, SAVE_FILENAME, this->save_prefix, base, page);
   
   return filename;
 }
@@ -989,69 +972,92 @@ static void pvr_event_handler (pvr_input_plugin_t *this) {
       /* make sure we are not paused */
       _x_set_speed(this->stream, XINE_SPEED_NORMAL);
 
-      if( v4l2_data->session_id != this->session ) {
-        /* if session changes -> closes the old one */
-        pthread_mutex_lock(&this->lock);
-        pvr_finish_recording(this);
-        time(&this->start_time);
-        this->show_time = this->start_time;
-        this->session = v4l2_data->session_id;
-        this->new_session = 1;
-        this->pvr_play_paused = 0;
-        this->scr_tunning = 0;
-        pvrscr_speed_tunning(this->scr, 1.0 );
-        pvr_break_rec_page(this);
-        pthread_mutex_unlock(&this->lock);
-        _x_demux_flush_engine (this->stream);
-      } else {
-        /* no session change, break the page and store a new show_time */
-        pthread_mutex_lock(&this->lock);
-        pvr_break_rec_page(this);
-        this->show_page = this->rec_page;
-        pthread_mutex_unlock(&this->lock);
-        time(&this->show_time);
+      if ( v4l2_data->session_id != -1) {
+        if( v4l2_data->session_id != this->session ) {
+          /* if session changes -> closes the old one */
+          pthread_mutex_lock(&this->lock);
+          pvr_finish_recording(this);
+          time(&this->start_time);
+          this->show_time = this->start_time;
+          this->session = v4l2_data->session_id;
+          this->new_session = 1;
+          this->pvr_play_paused = 0;
+          this->scr_tunning = 0;
+          pvrscr_speed_tunning(this->scr, 1.0 );
+          pvr_break_rec_page(this);
+          pthread_mutex_unlock(&this->lock);
+          _x_demux_flush_engine (this->stream);
+        } else {
+          /* no session change, break the page and store a new show_time */
+          pthread_mutex_lock(&this->lock);
+          pvr_break_rec_page(this);
+          this->show_page = this->rec_page;
+          pthread_mutex_unlock(&this->lock);
+          time(&this->show_time);
+        }
       }
-      
-      if( (v4l2_data->input != -1 && v4l2_data->input != this->input) ||
-          (v4l2_data->channel != -1 && v4l2_data->channel != this->channel) ||
-          (v4l2_data->frequency != -1 && v4l2_data->frequency != this->frequency) ) {
-        struct v4l2_frequency vf;
 
+      pthread_mutex_lock(&this->dev_lock);
+
+      /* change input */
+      if (v4l2_data->input != -1 && v4l2_data->input != this->input) {
         this->input = v4l2_data->input;
-        this->channel = v4l2_data->channel;
-        this->frequency = v4l2_data->frequency;
 
-        lprintf("switching to input:%d chan:%d freq:%.2f\n", 
-		v4l2_data->input, 
-		v4l2_data->channel,
-		(float)v4l2_data->frequency * 62.5);
-  
-        pthread_mutex_lock(&this->dev_lock);
-        if( ioctl(this->dev_fd, VIDIOC_S_INPUT, &this->input) )
-          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
-		  "input_pvr: error setting v4l2 input\n");
-          
-        vf.frequency = this->frequency;
-        vf.tuner = 0;
-        if( ioctl(this->dev_fd, VIDIOC_S_FREQUENCY, &vf) )
-          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
-		  "input_pvr: error setting v4l2 frequency\n");
-
-        /* workaround an ivtv bug where stream gets bad mpeg2 artifacts
-         * after changing inputs. reopening the device fixes it.
-         */
+        /* as of ivtv 0.10.6: must close and reopen to set input */
         close(this->dev_fd);
         this->dev_fd = open (this->class->devname, O_RDWR);
-        if (this->dev_fd == -1) {
-          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
-		  "input_pvr: error opening device %s\n", this->class->devname );
-          return;
+        if (this->dev_fd < 0) {
+          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+                  "input_pvr: error opening device %s\n", this->class->devname );
+        } else {
+          if( ioctl(this->dev_fd, VIDIOC_S_INPUT, &this->input) == 0 ) {
+            lprintf("Tuner Input set to:%d\n", v4l2_data->input);
+          } else {
+            xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG, 
+                    "input_pvr: error setting v4l2 input\n");
+          }
         }
-        pthread_mutex_unlock(&this->dev_lock);
-        
-        /* FIXME: also flush the device */
-        /* _x_demux_flush_engine(this->stream); */
       }
+
+      /* change channel */          
+      if (v4l2_data->channel != -1 && v4l2_data->channel != this->channel) {
+        lprintf("change channel to:%d\n", v4l2_data->channel);
+        this->channel = v4l2_data->channel;
+      }
+
+      /* change frequency */
+      if (v4l2_data->frequency != -1 && v4l2_data->frequency != this->frequency) {
+        double freq = (double)v4l2_data->frequency / 1000.0;
+        struct v4l2_frequency vf;
+        struct v4l2_tuner vt;
+        double fac = 16;
+
+        memset(&vf, 0, sizeof(vf));
+        memset(&vt, 0, sizeof(vt));
+
+        this->frequency = v4l2_data->frequency;
+
+        if (ioctl(this->dev_fd, VIDIOC_G_TUNER, &vt) == 0) {
+          fac = (vt.capability & V4L2_TUNER_CAP_LOW) ? 16000 : 16;
+        }
+
+        vf.tuner = 0;
+        vf.type = vt.type;
+        vf.frequency = (__u32)(freq * fac);
+
+        if (ioctl(this->dev_fd, VIDIOC_S_FREQUENCY, &vf) == 0) {
+          lprintf("Tuner Frequency set to %d (%f.3 MHz)\n", vf.frequency, vf.frequency / fac);
+        } else {
+          xprintf(this->stream->xine, XINE_VERBOSITY_DEBUG,
+                  "input_pvr: error setting v4l2 frequency\n");
+        }
+      }
+
+      pthread_mutex_unlock(&this->dev_lock);
+        
+      /* FIXME: also flush the device */
+      /* _x_demux_flush_engine(this->stream); */
+
       break;
     
     
@@ -1459,7 +1465,7 @@ static input_plugin_t *pvr_class_get_instance (input_class_t *cls_gen, xine_stre
   mrl = strdup(data);
   aux = &mrl[5];
 
-  this = (pvr_input_plugin_t *) xine_xmalloc (sizeof (pvr_input_plugin_t));
+  this = calloc(1, sizeof (pvr_input_plugin_t));
   this->class        = cls;
   this->stream       = stream;
   this->dev_fd       = -1;
@@ -1543,7 +1549,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   pvr_input_class_t  *this;
 
-  this = (pvr_input_class_t *) xine_xmalloc (sizeof (pvr_input_class_t));
+  this = calloc(1, sizeof (pvr_input_class_t));
 
   this->xine   = xine;
   this->config = xine->config;
