@@ -256,8 +256,8 @@ static xine_mrl_t **smb_class_get_dir (input_class_t *this_gen,
 	}
   
 	if ((dir = smbc_opendir(current_path_smb)) >= 0){
-		xine_mrl_t *dir_files  = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t) * MAXFILES);
-		xine_mrl_t *norm_files = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t) * MAXFILES);
+		xine_mrl_t *dir_files  = (xine_mrl_t *) calloc(MAXFILES, sizeof(xine_mrl_t));
+		xine_mrl_t *norm_files = (xine_mrl_t *) calloc(MAXFILES, sizeof(xine_mrl_t));
     		int num_dir_files=0;
 		int num_norm_files=0;
 		while ((pdirent = smbc_readdir(dir)) != NULL){
@@ -265,9 +265,7 @@ static xine_mrl_t **smb_class_get_dir (input_class_t *this_gen,
 				dir_files[num_dir_files].link   = NULL;
 				dir_files[num_dir_files].type = mrl_file | mrl_file_directory;
 				dir_files[num_dir_files].origin = strdup(current_path);
-				dir_files[num_dir_files].mrl    = (char *) xine_xmalloc(
-					strlen(current_path) + 1 + strlen(pdirent->name) + 1);
-				sprintf(dir_files[num_dir_files].mrl, "%s/%s", current_path, pdirent->name);
+				asprintf(&(dir_files[num_dir_files].mrl), "%s/%s", current_path, pdirent->name);
 				dir_files[num_dir_files].size   = pdirent->dirlen;
 				num_dir_files ++;
 			}else if (pdirent->smbc_type == SMBC_SERVER){  
@@ -275,17 +273,14 @@ static xine_mrl_t **smb_class_get_dir (input_class_t *this_gen,
 					dir_files[num_dir_files].link   = NULL;
 					dir_files[num_dir_files].type = mrl_file | mrl_file_directory;
 					dir_files[num_dir_files].origin = strdup("smb:/");
-					dir_files[num_dir_files].mrl    = (char *) xine_xmalloc(strlen("smb:/") + 4);
-					sprintf(dir_files[num_dir_files].mrl, "%s/%s", "smb:/", "..");
+					asprintf(&(dir_files[num_dir_files].mrl), "%s/%s", "smb:/", "..");
 					dir_files[num_dir_files].size   = pdirent->dirlen;
 					num_dir_files ++;
 				}
 				dir_files[num_dir_files].link   = NULL;
 				dir_files[num_dir_files].type   = mrl_file | mrl_file_directory;
 				dir_files[num_dir_files].origin = strdup("smb:/");
-				dir_files[num_dir_files].mrl    = 
-					(char *) xine_xmalloc(strlen("smb:/") + 1 + strlen(pdirent->name) + 1);
-				sprintf(dir_files[num_dir_files].mrl, "%s/%s", "smb:/", pdirent->name);
+				asprintf(*(dir_files[num_dir_files].mrl), "%s/%s", "smb:/", pdirent->name);
 				dir_files[num_dir_files].size   = pdirent->dirlen;
 				num_dir_files ++;
 			} else if (pdirent->smbc_type == SMBC_FILE_SHARE){
@@ -293,9 +288,7 @@ static xine_mrl_t **smb_class_get_dir (input_class_t *this_gen,
 					dir_files[num_dir_files].link   = NULL;
 					dir_files[num_dir_files].type   = mrl_file | mrl_file_directory;
 					dir_files[num_dir_files].origin = strdup(current_path);
-					dir_files[num_dir_files].mrl    = (char *) xine_xmalloc(
-						strlen(current_path) + 3);
-					sprintf(dir_files[num_dir_files].mrl, "%s/%s", current_path, "..");
+					asprintf(&(dir_files[num_dir_files].mrl), "%s/%s", current_path, "..");
 					dir_files[num_dir_files].type   |= mrl_file_directory;
 					dir_files[num_dir_files].size   = pdirent->dirlen;
 					num_dir_files ++;
@@ -304,9 +297,7 @@ static xine_mrl_t **smb_class_get_dir (input_class_t *this_gen,
 					dir_files[num_dir_files].link   = NULL;
 					dir_files[num_dir_files].type   = mrl_file | mrl_file_directory;
 					dir_files[num_dir_files].origin = strdup(current_path);
-					dir_files[num_dir_files].mrl    = (char *) xine_xmalloc(
-						strlen(current_path) + 1 + strlen(pdirent->name) + 1);
-					sprintf(dir_files[num_dir_files].mrl, "%s/%s", current_path, pdirent->name);
+					asprintf(&(dir_files[num_dir_files].mrl), "%s/%s", current_path, pdirent->name);
 					dir_files[num_dir_files].size   = pdirent->dirlen;
 					num_dir_files ++;
        				}
@@ -314,18 +305,14 @@ static xine_mrl_t **smb_class_get_dir (input_class_t *this_gen,
 				dir_files[num_dir_files].link   = NULL;
 				dir_files[num_dir_files].type   = mrl_file | mrl_file_directory;
 				dir_files[num_dir_files].origin = strdup(current_path);
-				dir_files[num_dir_files].mrl    = 
-					(char *) xine_xmalloc(strlen(current_path) + 1 + strlen(pdirent->name) + 1);
-				sprintf(dir_files[num_dir_files].mrl, "%s/%s", current_path, pdirent->name);
+				asprintf(&(dir_files[num_dir_files].mrl), "%s/%s", current_path, pdirent->name);
 				dir_files[num_dir_files].size   = pdirent->dirlen;
 				num_dir_files ++;
 			}else if (pdirent->smbc_type == SMBC_FILE){
 				norm_files[num_norm_files].link   = NULL;
 				norm_files[num_norm_files].type   = mrl_file | mrl_file_normal;
 				norm_files[num_norm_files].origin = strdup(current_path);
-				norm_files[num_norm_files].mrl    = 
-					(char *) xine_xmalloc(strlen(current_path) + 1 + strlen(pdirent->name) + 1);
-				sprintf(norm_files[num_norm_files].mrl, "%s/%s", current_path, pdirent->name);
+				asprintf(&(norm_files[num_norm_files].mrl), "%s/%s", current_path, pdirent->name);
 				norm_files[num_norm_files].size   = pdirent->dirlen;
 				num_norm_files ++;
 			}
@@ -335,8 +322,7 @@ static xine_mrl_t **smb_class_get_dir (input_class_t *this_gen,
 		if (num_dir_files == 0) {
 			dir_files[num_dir_files].link   = NULL;
 			dir_files[num_dir_files].origin = strdup(current_path);
-			dir_files[num_dir_files].mrl    = (char *) xine_xmalloc(strlen(current_path) + 4);
-			sprintf(dir_files[num_dir_files].mrl, "%s/%s", current_path, "..");
+			asprintf(&(dir_files[num_dir_files].mrl), "%s/%s", current_path, "..");
 			dir_files[num_dir_files].type = mrl_file | mrl_file_directory;
 			dir_files[num_dir_files].size   = 0;
 			num_dir_files ++;
@@ -359,7 +345,7 @@ static xine_mrl_t **smb_class_get_dir (input_class_t *this_gen,
 				++this->mrls_allocated_entries;
 				this->mrls = realloc(this->mrls, 
 					(this->mrls_allocated_entries+1) * sizeof(xine_mrl_t*));
-				this->mrls[num_files] = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t));
+				this->mrls[num_files] = calloc(1, sizeof(xine_mrl_t));
 			}else
 				memset(this->mrls[num_files], 0, sizeof(xine_mrl_t));
       
@@ -376,7 +362,7 @@ static xine_mrl_t **smb_class_get_dir (input_class_t *this_gen,
 				++this->mrls_allocated_entries;
 				this->mrls = realloc(this->mrls, 
 					(this->mrls_allocated_entries+1) * sizeof(xine_mrl_t*));
-				this->mrls[num_files] = (xine_mrl_t *) xine_xmalloc(sizeof(xine_mrl_t));
+				this->mrls[num_files] = calloc(1, sizeof(xine_mrl_t));
 			}else
 				memset(this->mrls[num_files], 0, sizeof(xine_mrl_t));
 
@@ -475,7 +461,7 @@ smb_class_get_instance (input_class_t *class_gen, xine_stream_t *stream,
 	if (strncmp (mrl, "smb://",6))
 		return NULL;
 
-	this = (smb_input_t *)xine_xmalloc(sizeof(smb_input_t));
+	this = calloc(1, sizeof(smb_input_t));
 	this->stream = stream;
 	this->mrl = strdup (mrl);
 	this->fd = -1;
@@ -514,7 +500,7 @@ static void
 	if (smbc_init(smb_auth,(xine->verbosity >= XINE_VERBOSITY_DEBUG)))
 	  goto _exit_error;
 	
-	this = (smb_input_class_t *) xine_xmalloc(sizeof(smb_input_class_t));
+	this = calloc(1, sizeof(smb_input_class_t));
 	this->xine = xine;
 
 	this->input_class.get_instance       = smb_class_get_instance;
