@@ -35,11 +35,12 @@
 #endif
 #include "win32.h"
 #include "driver.h"
+#include "ext.h"
 
 #ifdef EMU_QTX_API
 #include "wrapper.h"
-static int report_func(void *stack_base, int stack_size, reg386_t *reg, u_int32_t *flags);
-static int report_func_ret(void *stack_base, int stack_size, reg386_t *reg, u_int32_t *flags);
+int report_func(void *stack_base, int stack_size, reg386_t *reg, u_int32_t *flags);
+int report_func_ret(void *stack_base, int stack_size, reg386_t *reg, u_int32_t *flags);
 #endif
 
 //#undef TRACE
@@ -362,7 +363,6 @@ static WIN_BOOL MODULE_FreeLibrary( WINE_MODREF *wm )
 HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 {
 	WINE_MODREF *wm = 0;
-	char* listpath[] = { "", "", "/usr/lib/win32", "/usr/local/lib/win32", 0 };
 	char path[512];
 	char checked[2000];
         int i = -1;
@@ -380,31 +380,17 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 //	if(fs_installed==0)
 //	    install_fs();
 
-	while (wm == 0 && listpath[++i])
+	for (i = 0; !wm && (i < 2); i++)
 	{
 	    memset (&path, 0, sizeof (path));
 
-	    if (i < 2)
-	    {
-		if (i == 0)
-		    /* check just original file name */
-		    strncpy(path, libname, 511);
-                else
-		    /* check default user path */
-		    strncpy(path, win32_def_path, 300);
+	    if (i == 0) {
+	        /* check just original file name */
+	        strncpy(path, libname, sizeof(path) - 1);
+            } else {
+	        /* check default user path */
+                snprintf(path, sizeof(path), "%s/%s", win32_def_path, libname);
 	    }
-	    else if (strcmp(win32_def_path, listpath[i]))
-                /* path from the list */
-		strncpy(path, listpath[i], 300);
-	    else
-		continue;
-
-	    if (i > 0)
-	    {
-		strcat(path, "/");
-		strncat(path, libname, 100);
-	    }
-	    path[511] = 0;
 	    wm = MODULE_LoadLibraryExA( path, hfile, flags );
 
 	    if (!wm)
@@ -533,8 +519,6 @@ HMODULE WINAPI LoadLibraryExA(LPCSTR libname, HANDLE hfile, DWORD flags)
 	    printf ("wine/module: QuickTime.qts patched!!! old entry=%p\n",ptr[0]);
 
 #ifdef EMU_QTX_API
-	    report_entry = report_func;
-	    report_ret   = report_func_ret;
 	    wrapper_target=ptr[0];
 	    ptr[0]=wrapper;
 #endif
@@ -697,7 +681,7 @@ static int dump_component(char* name,int type,void* _orig, ComponentParameters *
 static u_int32_t ret_array[4096];
 static int ret_i=0;
 
-static int report_func(void *stack_base, int stack_size, reg386_t *reg, u_int32_t *flags)
+int report_func(void *stack_base, int stack_size, reg386_t *reg, u_int32_t *flags)
 {
 #ifdef DEBUG_QTX_API
   int i;
@@ -896,7 +880,7 @@ static int report_func(void *stack_base, int stack_size, reg386_t *reg, u_int32_
     return 0;
 }
 
-static int report_func_ret(void *stack_base, int stack_size, reg386_t *reg, u_int32_t *flags)
+int report_func_ret(void *stack_base, int stack_size, reg386_t *reg, u_int32_t *flags)
 {
 #ifdef DEBUG_QTX_API
   int i;
@@ -1011,8 +995,6 @@ FARPROC MODULE_GetProcAddress(
 //      || !strcmp(function,"_CallComponent")
       ){
 	fprintf(stderr,"theQuickTimeDispatcher caught -> %p\n",retproc);
-      report_entry = report_func;
-      report_ret   = report_func_ret;
       wrapper_target=(void *)retproc;
       retproc=(void *)wrapper;
     }
