@@ -91,7 +91,7 @@ static void fooaudio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
     this->channels = buf->decoder_info[3];
 
     /* initialize the data accumulation buffer */
-    this->buf = xine_xmalloc(AUDIOBUFSIZE);
+    this->buf = calloc(1, AUDIOBUFSIZE);
     this->bufsize = AUDIOBUFSIZE;
     this->size = 0;
 
@@ -255,7 +255,7 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
 
   fooaudio_decoder_t *this ;
 
-  this = (fooaudio_decoder_t *) xine_xmalloc (sizeof (fooaudio_decoder_t));
+  this = (fooaudio_decoder_t *) calloc(1, sizeof(fooaudio_decoder_t));
 
   /* connect the member functions */
   this->audio_decoder.decode_data         = fooaudio_decode_data;
