@@ -15,15 +15,13 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * foovideo.c: This is a reference video decoder for the xine multimedia
  * player. It really works too! It will output frames of packed YUY2 data
  * where each byte in the map is the same value, which is 3 larger than the
  * value from the last frame. This creates a slowly rotating solid color
  * frame when the frames are played in succession.
- *
- * $Id: foovideo.c,v 1.26 2006/07/10 22:08:43 dgp85 Exp $
  */
 
 #include <stdio.h>
@@ -208,7 +206,7 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, xine_stre
 
   foovideo_decoder_t  *this ;
 
-  this = (foovideo_decoder_t *) xine_xmalloc (sizeof (foovideo_decoder_t));
+  this = (foovideo_decoder_t *) calloc(1, sizeof(foovideo_decoder_t));
 
   this->video_decoder.decode_data         = foovideo_decode_data;
   this->video_decoder.flush               = foovideo_flush;
@@ -258,7 +256,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
   foovideo_class_t *this;
 
-  this = (foovideo_class_t *) xine_xmalloc (sizeof (foovideo_class_t));
+  this = (foovideo_class_t *) calloc(1, sizeof(foovideo_class_t));
 
   this->decoder_class.open_plugin     = open_plugin;
   this->decoder_class.get_identifier  = get_identifier;
