@@ -15,15 +15,12 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * Upmix audio filter for xine.
  *   (c) 2004 James Courtier-Dutton (James@superbug.demon.co.uk)
  * This is an up-mix audio filter post plugin.
  * It simply converts Mono into Stereo.
- *
- * $Id: upmix_mono.c,v 1.5 2006/01/27 07:46:12 tmattern Exp $
- *
  */
 
 #include <stdio.h>
@@ -195,32 +192,32 @@ static void upmix_mono_port_put_buffer(xine_audio_port_t *port_gen,
     _x_extra_info_merge(buf1->extra_info, buf->extra_info);
 
     {
-      int step = buf->format.bits / 8;
+      const size_t step = buf->format.bits / 8;
       uint8_t *src  = (uint8_t *)buf->mem;
       uint8_t *dst0 = (uint8_t *)buf0->mem;
       uint8_t *dst1 = (uint8_t *)buf1->mem;
 
-      int i, k;
+      int i;
       for (i = 0; i < buf->num_frames / 2; i++)
       {
-        for (k = 0; k < step; k++)
-          *dst0++ = *src++;
+	memcpy(dst0, src, step);
+	dst0 += step;
 
-        src -= step;
+	memcpy(dst0, src, step);
+	dst0 += step;
 
-        for (k = 0; k < step; k++)
-          *dst0++ = *src++;
+	src += step;
       }
 
       for (i = buf->num_frames / 2; i < buf->num_frames; i++)
       {
-        for (k = 0; k < step; k++)
-          *dst1++ = *src++;
+	memcpy(dst1, src, step);
+	dst1 += step;
 
-        src -= step;
+	memcpy(dst1, src, step);
+	dst1 += step;
 
-        for (k = 0; k < step; k++)
-          *dst1++ = *src++;
+	src += step;
       }
     }
 
@@ -247,11 +244,11 @@ static void upmix_mono_port_put_buffer(xine_audio_port_t *port_gen,
     _x_extra_info_merge(buf0->extra_info, buf->extra_info);
 
     {
-      int step = buf->format.bits / 8;
+      const size_t step = buf->format.bits / 8;
       uint8_t *src  = (uint8_t *)buf->mem;
       uint8_t *dst0 = (uint8_t *)buf0->mem;
       int cur_channel = this->params.channel;
-      int i, j, k;
+      int i, j;
       
       if( cur_channel >= this->channels )
         cur_channel = this->channels-1;
@@ -262,8 +259,8 @@ static void upmix_mono_port_put_buffer(xine_audio_port_t *port_gen,
       {
         for (j = 0; j < this->channels; j++ )
         {
-          for (k = 0; k < step; k++)
-            *dst0++ = *(src+k);
+	  memcpy(dst0, src, step);
+	  dst0 += step;
         }
         src += this->channels * step;
       }
@@ -296,7 +293,7 @@ static post_plugin_t *upmix_mono_open_plugin(post_class_t *class_gen, int inputs
                                              xine_audio_port_t **audio_target,
                                              xine_video_port_t **video_target)
 {
-  post_plugin_upmix_mono_t *this = (post_plugin_upmix_mono_t *)xine_xmalloc(sizeof(post_plugin_upmix_mono_t));
+  post_plugin_upmix_mono_t *this = calloc(1, sizeof(post_plugin_upmix_mono_t));
   post_in_t                *input;
   post_out_t               *output;
   xine_post_in_t       *input_api;
