@@ -148,7 +148,7 @@ static post_plugin_t *expand_open_plugin(post_class_t *class_gen, int inputs,
 					 xine_audio_port_t **audio_target,
 					 xine_video_port_t **video_target)
 {
-  post_expand_t     *this        = (post_expand_t *)xine_xmalloc(sizeof(post_expand_t));
+  post_expand_t     *this        = calloc(1, sizeof(post_expand_t));
   post_in_t         *input;
   xine_post_in_t    *input_param;
   post_out_t        *output;
