@@ -415,7 +415,7 @@ static post_plugin_t *fftgraph_open_plugin(post_class_t *class_gen, int inputs,
 					 xine_video_port_t **video_target)
 {
   post_class_fftgraph_t  *class = (post_class_fftgraph_t *)class_gen;
-  post_plugin_fftgraph_t *this  = (post_plugin_fftgraph_t *)xine_xmalloc(sizeof(post_plugin_fftgraph_t));
+  post_plugin_fftgraph_t *this  = calloc(1, sizeof(post_plugin_fftgraph_t));
   post_in_t              *input;
   post_out_t             *output;
   post_out_t             *outputv;
