@@ -435,7 +435,7 @@ static post_plugin_t *fftscope_open_plugin(post_class_t *class_gen, int inputs,
 					 xine_audio_port_t **audio_target,
 					 xine_video_port_t **video_target)
 {
-  post_plugin_fftscope_t *this  = (post_plugin_fftscope_t *)xine_xmalloc(sizeof(post_plugin_fftscope_t));
+  post_plugin_fftscope_t *this  = calloc(1, sizeof(post_plugin_fftscope_t));
   post_class_fftscope_t  *class = (post_class_fftscope_t *)class_gen;
   post_in_t              *input;
   post_out_t             *output;
