@@ -15,10 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
- * $Id: video_out_vidix.c,v 1.74 2006/07/10 22:08:44 dgp85 Exp $
- * 
  * video_out_vidix.c
  *
  * xine video_out driver to vidix library by Miguel Freitas 30/05/2002
@@ -26,7 +24,6 @@
  * based on video_out_xv.c, video_out_syncfb.c and video_out_pgx64.c
  *
  * some vidix specific code from mplayer (file vosub_vidix.c)
- *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -373,7 +370,7 @@ static vo_frame_t *vidix_alloc_frame (vo_driver_t *this_gen) {
   /* vidix_driver_t  *this = (vidix_driver_t *) this_gen; */
   vidix_frame_t   *frame ;
 
-  frame = (vidix_frame_t *) xine_xmalloc (sizeof (vidix_frame_t));
+  frame = (vidix_frame_t *) calloc(1, sizeof(vidix_frame_t));
   if (!frame)
     return NULL;
 
@@ -703,6 +700,18 @@ static int vidix_get_property (vo_driver_t *this_gen, int property) {
     case VO_PROP_WINDOW_HEIGHT:
       this->props[property].value = this->sc.gui_height;
       break;
+    case VO_PROP_OUTPUT_WIDTH:
+      this->props[property].value = this->sc.output_width;
+      break;
+    case VO_PROP_OUTPUT_HEIGHT:
+      this->props[property].value = this->sc.output_height;
+      break;
+    case VO_PROP_OUTPUT_XOFFSET:
+      this->props[property].value = this->sc.output_xoffset;
+      break;
+    case VO_PROP_OUTPUT_YOFFSET:
+      this->props[property].value = this->sc.output_yoffset;
+      break;
   }
   
   lprintf ("video_out_vidix: property #%d = %d\n", property,
@@ -938,7 +947,7 @@ static vidix_driver_t *open_plugin (video_driver_class_t *class_gen) {
   vidix_driver_t       *this;
   int                   err;
     
-  this = (vidix_driver_t *) xine_xmalloc (sizeof (vidix_driver_t));
+  this = (vidix_driver_t *) calloc(1, sizeof(vidix_driver_t));
   if (!this)
     return NULL;
 
@@ -1080,7 +1089,7 @@ static void *init_class (xine_t *xine, void *visual_gen) {
   vidix_class_t        *this;
   int                   err;
   
-  this = (vidix_class_t *) xine_xmalloc (sizeof (vidix_class_t));
+  this = (vidix_class_t *) calloc(1, sizeof(vidix_class_t));
   if (!this)
     return NULL;
   
@@ -1235,7 +1244,7 @@ static vo_driver_t *vidixfb_open_plugin (video_driver_class_t *class_gen, const
   this->visual_type = XINE_VISUAL_TYPE_FB;
   
   /* Register config option for fb device */
-  device = config->register_string(config, "video.device.vidixfb_device", "/dev/fb0",
+  device = config->register_filename(config, "video.device.vidixfb_device", "/dev/fb0", XINE_CONFIG_STRING_IS_DEVICE_NAME,
     _("framebuffer device name"),
     _("Specifies the file name for the framebuffer device to be used.\n"
       "This setting is security critical, because when changed to a different file, xine "
