@@ -15,10 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: video_decoder.c,v 1.163 2007/02/20 00:34:58 dgp85 Exp $
- *
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,6 +25,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 
 #define XINE_ENGINE_INTERNAL
 
@@ -110,6 +108,15 @@ static void *video_decoder_loop (void *stream_gen) {
   int              prof_video_decode = -1;
   int              prof_spu_decode = -1;
   uint32_t         buftype_unknown = 0;
+
+#ifndef WIN32
+  /* nice(-value) will fail silently for normal users.
+   * however when running as root this may provide smoother
+   * playback. follow the link for more information:
+   * http://cambuca.ldhs.cetuc.puc-rio.br/~miguel/multimedia_sim/
+   */
+  nice(-1);
+#endif /* WIN32 */
   
   if (prof_video_decode == -1)
     prof_video_decode = xine_profiler_allocate_slot ("video decoder");
